<?php

namespace PyvioPaySdk\lib;

use Exception;
use ReflectionClass;
use ReflectionException;

class PyvioPayClient extends PyvioPay
{
    /**
     * @var string $merChantKey
     */
    protected $merChantKey = "default";

    public function __construct($merChantKey = 'default')
    {
        $this->merChantKey = $merChantKey;
    }

    /**
     * 请求接口
     *
     * @param object | array $request 请求实例对象 | 请求参数数组
     * @return PyvioPayRequest|null
     */
    public function request($request)
    {
        try {
            if ($request->method == 'GET') {
                return $this->getRequest($request);
            }

            if ($request->method == 'POST') {
                return $this->postRequest($request);
            }

            return $this->postRequest($request);
        } catch (Exception $e) {
            echo $e->getMessage();
            return null;
        }
    }

    /**
     * post请求接口
     *
     * @param object | array $request 请求实例对象 | 请求参数数组
     * @return PyvioPayRequest|null
     */
    public function postRequest($request)
    {
        try {
            // 请求接口获取应答
            if (is_object($request)) {
                $data = $this->objectToArray($request);
            } else {
                $data = $request;
            }

            return $this->post($data['funcCode'], $data['params'], $this->merChantKey);
        } catch (Exception $e) {
            echo $e->getMessage();
            return null;
        }
    }

    /**
     * get请求接口
     *
     * @param object | array $request 请求实例对象 | 请求参数数组
     * @return PyvioPayRequest|null
     */
    public function getRequest($request)
    {
        try {
            // 请求接口获取应答
            if (is_object($request)) {
                $data = $this->objectToArray($request);
            } else {
                $data = $request;
            }

            return $this->get($data['funcCode'], $data['params'], $this->merChantKey);

        } catch (Exception $e) {
            echo $e->getMessage();
            return null;
        }
    }

    /**
     *  对象转换数组
     * @param object $object 待转化参数对象
     * @return array
     * @throws ReflectionException
     */
    public function objectToArray($object)
    {
        $class = new ReflectionClass($object);
        $properties = $class->getProperties();
        $arrayData = array();
        $extendInfos = array();

        // 转换接口参数
        foreach ($properties as $key => $value) {
            $attrName = $value->getName();
            $method = 'get'.ucfirst($attrName);
            if ($class->hasMethod($method)) {
                $attrValue = $class->getMethod($method)->invoke($object);
                if (isset($attrValue)) {
                    if ($attrName != "extendInfos") {
                        $newKey = strtolower(preg_replace("/([a-z])([A-Z])/", "$1_$2", $attrName));
                        $arrayData[$newKey] = $attrValue;
                    } else {
                        $extendInfos = $attrValue;
                    }
                }
            }
        }

        // 获取接口方法
        $funcCode = $class->getMethod('getFunctionCode')->invoke($object);

        return array(
            'params'   => array_merge($arrayData, $extendInfos),
            'funcCode' => $funcCode ?: '',
        );
    }

}
