<?php

namespace PyvioPaySdk\lib;

use Exception;
use PyvioPaySdk\config\MerConfig;

class PyvioPayRequest
{
    # 请求头
    private $httpHeaders;
    # 网络应答码
    private $httpStateCode;
    # 请求URL
    private $url;
    # 请求数据
    private $requestParams;
    # 应答数据
    private $response;

    # 请求报错
    private $error = null;

    function curlRequest(MerConfig $merConfig, $url, $method = 'GET', $data = null, $headers = array())
    {
        try {
            $ch = curl_init();
            // 设置请求的URL
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_HEADER, false);
            curl_setopt($ch, CURLOPT_FAILONERROR, false);

            $timeStamp = '';

            $headers[] = 'SdkVersion: '.SDK_VERSION;
            $headers[] = 'charset: UTF-8';

            if (isset($data['headers']) && is_array($data['headers'])) {
                foreach ($data['headers'] as $k => $v) {
                    $headers[] = $k.': '.$v;

                    if ($k == 'X-Timestamp') {
                        $timeStamp = $v;
                    }
                }
                unset($data['headers']);
            }

            $jsonData = json_encode($data);

            // 设置请求方法
            if ($method === 'POST') {

                $signData = $merConfig->app_id.$timeStamp.$jsonData;

                $sign = PyvioPayTools::sha_with_rsa_sign($signData, $merConfig->rsa_private_key);
                $headers[] = 'Sign: '.$sign;

                curl_setopt($ch, CURLOPT_POST, true);
                if (!empty($data)) {
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonData);
                    $headers[] = 'Content-Length: '.strlen($jsonData);
                    $this->requestParams = $jsonData;
                }

            } elseif ($method === 'GET') {
                curl_setopt($ch, CURLOPT_HTTPGET, true);
            }

            // 设置自定义请求头
            if (!empty($headers)) {
                curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
            }

            // 设置返回结果不直接输出
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

            $this->url = $url;
            $this->httpHeaders = $headers;

            # 执行网络请求
            $resultString = curl_exec($ch);

            # 处理系统的报错
            if (curl_errno($ch)) {
                # 这里报错直接就关闭本次会话return了
                curl_close($ch);
                throw new Exception ("curl error: ".curl_error($ch));
            }

            # http应答码
            $this->httpStateCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

            # 关闭本次会话
            curl_close($ch);

            $this->response = json_decode($resultString, true);

            # 应答码异常情况
            if ($this->httpStateCode < 200 || $this->httpStateCode >= 400) {
                throw new Exception ("请求失败: HTTP_STATE_CODE-".$this->httpStateCode);
            }
        } catch (Exception $e) {
            $this->error = $e->getMessage();
        } finally {
            $log = sprintf("curl请求数据：\n 请求地址: %s \n 请求方式: %s \n 请求头: %s \n 请求体: %s \n 返回状态: %s \n 返回值: %s \n",
                $url,
                $method,
                json_encode($this->httpHeaders, JSON_UNESCAPED_UNICODE),
                json_encode($this->requestParams, JSON_UNESCAPED_UNICODE),
                $this->httpStateCode,
                json_encode($this->response, true),
            );
            PyvioPay::writeLog($log);

            return $this;
        }
    }

    /**
     * 本次请求的http头信息
     */
    public function getHttpHeaders()
    {
        return $this->httpHeaders;
    }

    /**
     * 本次请求应答的 http状态码
     */
    public function getHttpStateCode()
    {
        return $this->httpStateCode;
    }

    /**
     * 本次请求的数据
     */
    public function getRequestParams()
    {
        return $this->requestParams;
    }

    /**
     * 本次请求的应答数据
     */
    public function getResponse()
    {
        return $this->response;
    }

    /**
     * 获取请求执行失败信息
     */
    public function getErrorInfo()
    {
        return $this->error;
    }

    /**
     * 请求是否成功执行
     */
    public function isError()
    {
        return $this->error != null;
    }

    /**
     * 获取请求的URL
     */
    public function getUrl()
    {
        return $this->url;
    }
}
