<?php

namespace PyvioPaySdk\lib;

class PyvioPayTools
{
    /**
     * 私钥加签
     *
     * @param string $data 原数据(json字符串)
     * @param string $rsaPrivateKey 私钥
     * @param int $alg 默认 OPENSSL_ALGO_SHA256
     *
     * @return string 签名串
     */
    public static function sha_with_rsa_sign($data, $rsaPrivateKey, $alg = OPENSSL_ALGO_SHA256)
    {
        // 使用str_replace去掉指定的文本和空格
        $rsaPrivateKey = str_replace( array(
            '-----BEGIN PRIVATE KEY-----',
            '-----END PRIVATE KEY-----',
            " ",
            "\n",
            "\r"
        ), "", $rsaPrivateKey );

        $key = "-----BEGIN PRIVATE KEY-----\n".wordwrap($rsaPrivateKey, 64, "\n", true)."\n-----END PRIVATE KEY-----";
        $signature = '';
        try {
            openssl_sign($data, $signature, $key, $alg);
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
        return base64_encode($signature);
    }


    public static function checkEmpty($value)
    {
        return !isset($value) || trim($value) === "";
    }

    public static function endWith($str, $suffix)
    {
        $length = strlen($suffix);
        if ($length == 0) {
            return false;
        }
        return (substr($str, -$length) === $suffix);
    }

    /**
     * 校验 webhook 返回报文签名
     *
     * @param string $sign
     * @param string $content
     * @param string $publicKey
     * @return true|false 验证结果：成功/失败
     */
    public static function verify_webhook_sign($sign, $content, $publicKey)
    {
        try {
            $pubKey = self::getPublicKeyFromX509($publicKey);
            $result = openssl_verify($content, base64_decode($sign), $pubKey, OPENSSL_ALGO_SHA256);
            openssl_free_key($pubKey);
            return $result === 1;
        } catch (\Exception $e) {
            return false;
        }
    }

    /**
     * 从X509格式的公钥字符串中获取PublicKey对象
     *
     * @param string $publicKey
     * @return bool|resource
     * @throws \Exception
     */
    private static function getPublicKeyFromX509($publicKey)
    {
        $key = "-----BEGIN PUBLIC KEY-----\n";
        $key .= $publicKey."\n";
        $key .= "-----END PUBLIC KEY-----";
        return openssl_pkey_get_public($key);
    }
}
